/*
 * Copyright (c) 2024 DTB <trinity@trinity.moe>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <stdio.h> /* fclose(3), feof(3), fgetc(3), fgets(3), fopen(3),
                    * fprintf(3), fputc(3), stderr, stdin, stdout, EOF, FILE,
                    * NULL */
#include <stdlib.h> /* size_t */
#include <string.h> /* strchr */

#if !defined EX_OK || !defined EX_SOFTWARE || !defined EX_USAGE 
#	include <sysexits.h>
#endif

/* Done while looking at plan9ports' src/cmd/p.c.
 * Taken from plan9 are the interface (minus p(1)'s shell escaping and the
 * ability to page multiple files) and the practice of reading /dev/tty to get
 * keyboard input without going through stdin. Also, a newline is now printed
 * before querying the keyboard. */

static size_t l = 22; /* plan9 default */

static char *program_name = "pg";

static int print_lines(FILE *f, size_t l){
	int c;

	while((c = fgetc(f)) != EOF)
		if((c = fputc(c, stdout)) == EOF || (l -= (c == '\n')) == 0)
			break;

	return c;
}

int main(int argc, char *argv[]){
	unsigned char cmd[99+1];
	char *p;
	FILE *t;

	if(argc == 0)
		argv = &program_name;

	if(argc > 1){
		fprintf(stderr, "Usage: %s\n", argv[0]);
		return EX_USAGE;
	}

	if((t = fopen("/dev/tty", "rb")) == NULL){
		fprintf(stderr, "%s: Could not open /dev/tty\n", argv[0]);
		return EX_SOFTWARE;
	}

	while(print_lines(stdin, l) != EOF){
		fgets((char *)cmd, (sizeof cmd) / (sizeof *cmd), t);
		for(p = (char *)cmd; *p != '\0' && strchr(" \t", *p) != NULL; ++p);
		if(feof(t) || (*p | 32) == 'q')
			break;
	}

exit:	fclose(t);
	return EX_OK;
}
